---
created_at: '2013-04-09T16:23:17.000Z'
title: Teach Yourself Programming in Ten Years (2001)
url: http://norvig.com/21-days.html?
author: llambda
points: 181
story_text: ''
comment_text: 
num_comments: 90
story_id: 
story_title: 
story_url: 
parent_id: 
created_at_i: 1365524597
_tags:
- story
- author_llambda
- story_5519158
objectID: '5519158'

---
[Source](http://norvig.com/21-days.html? "Permalink to Teach Yourself Programming in Ten Years")

# Teach Yourself Programming in Ten Years

# Teach Yourself Programming in Ten Years

### Peter Norvig

| ----- |
| 

## Why is everyone in such a rush?

Walk into any bookstore, and you'll see how to _Teach Yourself Java in 24 Hours_ alongside endless variations offering to teach C, SQL, Ruby, Algorithms, and so on in a few days or hours. The Amazon advanced search for [[title: teach, yourself, hours, since: 2000][1] and found 512 such books. Of the top ten, nine are programming books (the other is about bookkeeping). Similar results come from replacing "teach yourself" with "learn" or "hours" with "days." 

The conclusion is that either people are in a big rush to learn about programming, or that programming is somehow fabulously easier to learn than anything else. Felleisen _et al._ give a nod to this trend in their book [_How to Design Programs][2]_, when they say "Bad programming is easy. _Idiots_ can learn it in _21 days_, even if they are _dummies_." The Abtruse Goose comic also had [their take][3]. 

Let's analyze what a title like [_Teach Yourself C++ in 24 Hours][4]_ could mean: 
* **Teach Yourself:** In 24 hours you won't have time to write several significant programs, and learn from your successes and failures with them. You won't have time to work with an experienced programmer and understand what it is like to live in a C++ environment. In short, you won't have time to learn much. So the book can only be talking about a superficial familiarity, not a deep understanding. As Alexander Pope said, a little learning is a dangerous thing.
* **C++:** In 24 hours you might be able to learn some of the syntax of C++ (if you already know another language), but you couldn't learn much about how to use the language. In short, if you were, say, a Basic programmer, you could learn to write programs in the style of Basic using C++ syntax, but you couldn't learn what C++ is actually good (and bad) for. So what's the point? [Alan Perlis][5] once said: "A language that doesn't affect the way you think about programming, is not worth knowing". One possible point is that you have to learn a tiny bit of C++ (or more likely, something like JavaScript or Processing) because you need to interface with an existing tool to accomplish a specific task. But then you're not learning how to program; you're learning to accomplish that task.
* **in 24 Hours:** Unfortunately, this is not enough, as the next section shows. 

## Teach Yourself Programming in Ten Years

Researchers ([Bloom (1985)][6], [Bryan & Harter (1899)][7], [Hayes (1989)][8], [Simmon & Chase (1973)][9]) have shown it takes about ten years to develop expertise in any of a wide variety of areas, including chess playing, music composition, telegraph operation, painting, piano playing, swimming, tennis, and research in neuropsychology and topology. The key is _deliberative_ practice: not just doing it again and again, but challenging yourself with a task that is just beyond your current ability, trying it, analyzing your performance while and after doing it, and correcting any mistakes. Then repeat. And repeat again. There appear to be no real shortcuts: even Mozart, who was a musical prodigy at age 4, took 13 more years before he began to produce world-class music. In another genre, the Beatles seemed to burst onto the scene with a string of #1 hits and an appearance on the Ed Sullivan show in 1964. But they had been playing small clubs in Liverpool and Hamburg since 1957, and while they had mass appeal early on, their first great critical success, _Sgt. Peppers_, was released in 1967. 

[Malcolm Gladwell][10] has popularized the idea, although he concentrates on 10,000 hours, not 10 years. Henri Cartier-Bresson (1908-2004) had another metric: "Your first 10,000 photographs are your worst." (He didn't anticipate that with digital cameras, some people can reach that mark in a week.) True expertise may take a lifetime: Samuel Johnson (1709-1784) said "Excellence in any department can be attained only by the labor of a lifetime; it is not to be purchased at a lesser price." And Chaucer (1340-1400) complained "the lyf so short, the craft so long to lerne." Hippocrates (c. 400BC) is known for the excerpt "ars longa, vita brevis", which is part of the longer quotation "Ars longa, vita brevis, occasio praeceps, experimentum periculosum, iudicium difficile", which in English renders as "Life is short, [the] craft long, opportunity fleeting, experiment treacherous, judgment difficult."  Of course, no single number can be the final answer: it doesn't seem reasonable to assume that all skills (e.g., programming, chess playing, checkers playing, and music playing) could all require exactly the same amount of time to master, nor that all people will take exactly the same amount of time. As Prof. [ K. Anders Ericsson][11] puts it, "In most domains it's remarkable how much time even the most talented individuals need in order to reach the highest levels of performance. The 10,000 hour number just gives you a sense that we're talking years of 10 to 20 hours a week which those who some people would argue are the most innately talented individuals still need to get to the highest level." 

## So You Want to be a Programmer

Here's my recipe for programming success: 

* Get **interested** in programming, and do some because it is fun. Make sure that it keeps being enough fun so that you will be willing to put in your ten years/10,000 hours.
* **Program**. The best kind of learning is [learning by doing][12]. To put it more technically, "the maximal level of performance for individuals in a given domain is not attained automatically as a function of extended experience, but the level of performance can be increased even by highly experienced individuals as a result of deliberate efforts to improve." [(p. 366)][13] and "the most effective learning requires a well-defined task with an appropriate difficulty level for the particular individual, informative feedback, and opportunities for repetition and corrections of errors." (p. 20-21) The book _ [Cognition in Practice: Mind, Mathematics, and Culture in Everyday Life][14]_ is an interesting reference for this viewpoint.
* **Talk with** other programmers; read other programs. This is more important than any book or training course.
* If you want, put in four years at a **college** (or more at a graduate school). This will give you access to some jobs that require credentials, and it will give you a deeper understanding of the field, but if you don't enjoy school, you can (with some dedication) get similar experience on your own or on the job. In any case, book learning alone won't be enough. "Computer science education cannot make anybody an expert programmer any more than studying brushes and pigment can make somebody an expert painter" says Eric Raymond, author of _The New Hacker's Dictionary_. One of the best programmers I ever hired had only a High School degree; he's produced a lot of [great][15] [software][16], has his own [news group][17], and made enough in stock options to buy his own [nightclub][18].
* Work on **projects with** other programmers. Be the best programmer on some projects; be the worst on some others. When you're the best, you get to test your abilities to lead a project, and to inspire others with your vision. When you're the worst, you learn what the masters do, and you learn what they don't like to do (because they make you do it for them).
* Work on **projects _after_** other programmers. Understand a program written by someone else. See what it takes to understand and fix it when the original programmers are not around. Think about how to design your programs to make it easier for those who will maintain them after you.
* Learn at least a half dozen **programming languages**. Include one language that emphasizes class abstractions (like Java or C++), one that emphasizes functional abstraction (like Lisp or ML or Haskell), one that supports syntactic abstraction (like Lisp), one that supports declarative specifications (like Prolog or C++ templates), and one that emphasizes parallelism (like Clojure or Go). 
* Remember that there is a "**computer**" in "computer science". Know how long it takes your computer to execute an instruction, fetch a word from memory (with and without a cache miss), read consecutive words from disk, and seek to a new location on disk. ([Answers here.][19]) 
* Get involved in a language **standardization** effort. It could be the ANSI C++ committee, or it could be deciding if your local coding style will have 2 or 4 space indentation levels. Either way, you learn about what other people like in a language, how deeply they feel so, and perhaps even a little about why they feel so.
* Have the good sense to **get off** the language standardization effort as quickly as possible.  With all that in mind, its questionable how far you can get just by book learning. Before my first child was born, I read all the _How To_ books, and still felt like a clueless novice. 30 Months later, when my second child was due, did I go back to the books for a refresher? No. Instead, I relied on my personal experience, which turned out to be far more useful and reassuring to me than the thousands of pages written by experts. 

Fred Brooks, in his essay [_No Silver Bullet][20]_ identified a three-part plan for finding great software designers: 

1. Systematically identify top designers as early as possible.

2. Assign a career mentor to be responsible for the development of the prospect and carefully keep a career file.

3. Provide opportunities for growing designers to interact and stimulate each other.

This assumes that some people already have the qualities necessary for being a great designer; the job is to properly coax them along. [Alan Perlis][5] put it more succinctly: "Everyone can be taught to sculpt: Michelangelo would have had to be taught how not to. So it is with the great programmers". Perlis is saying that the greats have some internal quality that transcends their training. But where does the quality come from? Is it innate? Or do they develop it through diligence? As Auguste Gusteau (the fictional chef in _Ratatouille_) puts it, "anyone can cook, but only the fearless can be great." I think of it more as willingness to devote a large portion of one's life to deliberative practice. But maybe _fearless_ is a way to summarize that. Or, as Gusteau's critic, Anton Ego, says: "Not everyone can become a great artist, but a great artist can come from anywhere." 

So go ahead and buy that Java/Ruby/Javascript/PHP book; you'll probably get some use out of it. But you won't change your life, or your real overall expertise as a programmer in 24 hours or 21 days. How about working hard to continually improve over 24 months? Well, now you're starting to get somewhere... 

* * *

## References

Bloom, Benjamin (ed.) [_Developing Talent in Young People][21]_, Ballantine, 1985. 

Brooks, Fred, [_No Silver Bullets][22]_, IEEE Computer, vol. 20, no. 4, 1987, p. 10-19. 

Bryan, W.L. & Harter, N. "Studies on the telegraphic language: The acquisition of a hierarchy of habits. _Psychology Review_, 1899, 8, 345-375

Hayes, John R., [_Complete Problem Solver][8]_ Lawrence Erlbaum, 1989. 

Chase, William G. & Simon, Herbert A. ["Perception in Chess"][23] _Cognitive Psychology_, 1973, 4, 55-81.

Lave, Jean, [_Cognition in Practice: Mind, Mathematics, and Culture in Everyday Life][14]_, Cambridge University Press, 1988. 

* * *

## Answers

Approximate timing for various operations on a typical PC:

| execute typical instruction 1/1,000,000,000 sec = 1 nanosec 
| fetch from L1 cache memory 0.5 nanosec 
| branch misprediction 5 nanosec 
| fetch from L2 cache memory 7 nanosec 
| Mutex lock/unlock 25 nanosec 
| fetch from main memory 100 nanosec 
| send 2K bytes over 1Gbps network 20,000 nanosec 
| read 1MB sequentially from memory 250,000 nanosec 
| fetch from new disk location (seek) 8,000,000 nanosec 
| read 1MB sequentially from disk 20,000,000 nanosec 
| send packet US to Europe and back 150 milliseconds = 150,000,000 nanosec 

* * *

## Appendix: Language Choice

Several people have asked what programming language they should learn first. There is no one answer, but consider these points: 

* _Use your friends_. When asked "what operating system should I use, Windows, Unix, or Mac?", my answer is usually: "use whatever your friends use." The advantage you get from learning from your friends will offset any intrinsic difference between OS, or between programming languages. Also consider your future friends: the community of programmers that you will be a part of if you continue. Does your chosen language have a large growing community or a small dying one? Are there books, web sites, and online forums to get answers from? Do you like the people in those forums? 
* _Keep it simple_. Programming languages such as C++ and Java are designed for professional development by large teams of experienced programmers who are concerned about the run-time efficiency of their code. As a result, these languages have complicated parts designed for these circumstances. You're concerned with learning to program. You don't need that complication. You want a language that was designed to be easy to learn and remember by a single new programmer. 
* _Play._ Which way would you rather learn to play the piano: the normal, interactive way, in which you hear each note as soon as you hit a key, or "batch" mode, in which you only hear the notes after you finish a whole song? Clearly, interactive mode makes learning easier for the piano, and also for programming. Insist on a language with an interactive mode and use it.  Given these criteria, my recommendations for a first programming language would be [**Python][24]** or [**Scheme][25]**. Another choice is Javascript, not because it is perfectly well-designed for beginners, but because there are so many online tutorials for it, such as [Khan Academy's tutorial][26]. But your circumstances may vary, and there are other good choices. If your age is a single-digit, you might prefer [Alice][27] or [Squeak][28] or [Blockly][29] (older learners might also enjoy these). The important thing is that you choose and get started. 
* * *

## Appendix: Books and Other Resources

Several people have asked what books and web pages they should learn from. I repeat that "book learning alone won't be enough" but I can recommend the following: 

* **Scheme:** [Structure and Interpretation of Computer Programs (Abelson & Sussman)][30] is probably the best introduction to computer science, and it does teach programming as a way of understanding the computer science. You can see [online videos of lectures][31] on this book, as well as the [complete text online][32]. The book is challenging and will weed out some people who perhaps could be successful with another approach. 
* **Scheme:** [How to Design Programs (Felleisen _et al._)][33] is one of the best books on how to actually design programs in an elegant and functional way. 
* **Python:** [Python Programming: An Intro to CS (Zelle)][34] is a good introduction using Python. 
* **Python:** Several online [tutorials][35] are available at [Python.org][24]. 
* **Oz:** [Concepts, Techniques, and Models of Computer Programming (Van Roy & Haridi)][36] is seen by some as the modern-day successor to Abelson & Sussman. It is a tour through the big ideas of programming, covering a wider range than Abelson & Sussman while being perhaps easier to read and follow. It uses a language, Oz, that is not widely known but serves as a basis for learning other languages. <
* * *

## Notes

T. Capey points out that the [Complete Problem Solver][8] page on Amazon now has the "Teach Yourself Bengali in 21 days" and "Teach Yourself Grammar and Style" books under the "Customers who shopped for this item also shopped for these items" section. I guess that a large portion of the people who look at that book are coming from this page. Thanks to Ross Cohen for help with Hippocrates. 

          

## Translations

Thanks to the following authors, translations of this page are available in: 

* * *

[Arabic  
(Mohamed A. Yahya)   
![][37]

* * *

[Bulgarian  
(Boyko Bantchev)  
![][38]

* * *

[Chinese   
(Xiaogang Guo)   
![][39]][40]

* * *

[Croatian   
(Tvrtko Bedekovic)   
![][41]][42]

* * *

[Esperanto   
(Federico Gobbo)   
![][43] ][44]

* * *

[French   
(Etienne Beauchesne)   
![][45] ][46]

* * *

[German   
(Stefan Ram)   
![][47]][48]

* * *

[Hebrew   
(Eric McCain)   
![][49]][50]

* * *

[Hindi  
(Vikash Tiwari)   
![][51]][52]

* * *

[Hungarian  
(Marton Mestyan)   
![][53]][54]

* * *

[Indonesian  
(Tridjito Santoso)   
![][55]][56]

* * *

[Italian  
(Fabio Z. Tessitore)   
![][57]][58]

* * *

[Japanese   
(yomoyomo)   
![][59]][60]

* * *

[Korean (John Hwang)   
![][61]][62]

* * *

[Persian  
(Mehdi Asgari)   
![][63]][64]

* * *

[Polish  
(Kuba Nowak)   
![][65]][66]

* * *

[Portuguese  
(Augusto Radtke)   
![][67]][68]

* * *

[Romanian  
(Ştefan Lazăr)   
![][69]][70]

* * *

[Russian  
(Konstantin Ptitsyn)   
![][71]][72]

* * *

[Serbian  
(Lazar Kovacevic)   
![][73]][74]

* * *

[Spanish   
(Carlos Rueda)   
![][75]][76]

* * *

[Slovak  
(Jan Waclawek)   
![][77]][78]

* * *

[Turkish   
(Çağıl Uluşahin)   
![][79] ][80]

* * *

[Ukranian  
(Oleksii Molchanovskyi)   
![][81] ][82]

* * *
* * *

[_Peter Norvig_ (Copyright 2001—2014)][83]

* * *

[View the forum thread.][84]

[1]: http://www.amazon.com/gp/search/ref=sr_adv_b/?search-alias=stripbooks&unfiltered=1&field-keywords=&field-author=&field-title=teach+yourself+hours&field-isbn=&field-publisher=&node=&field-p_n_condition-type=&field-feature_browse-bin=&field-subject=&field-language=&field-dateop=After&field-datemod=&field-dateyear=2000&sort=relevanceexprank&Adv-Srch-Books-Submit.x=16&Adv-Srch-Books-Submit.y=5
[2]: http://www.ccs.neu.edu/home/matthias/HtDP2e/index.html
[3]: http://abstrusegoose.com/249
[4]: http://www.amazon.com/Sams-Teach-Yourself-Hours-5th/dp/0672333317/ref=sr_1_6?s=books&ie=UTF8&qid=1412708443&sr=1-6&keywords=learn+c%2B%2B+days
[5]: http://www-pu.informatik.uni-tuebingen.de/users/klaeren/epigrams.html
[6]: http://www.amazon.com/exec/obidos/ASIN/034531509X/
[7]: http://norvig.com#bh
[8]: http://www.amazon.com/exec/obidos/ASIN/0805803092
[9]: http://norvig.com#sc
[10]: http://www.amazon.com/Outliers-Story-Success-Malcolm-Gladwell/dp/0316017922
[11]: http://www.amazon.com/K.-Anders-Ericsson/e/B000APB8AQ/ref=dp_byline_cont_book_1
[12]: http://www.engines4ed.org/hyperbook/nodes/NODE-120-pg.html
[13]: http://www2.umassd.edu/swpi/DesignInCS/expertise.html
[14]: http://www.amazon.com/exec/obidos/ASIN/0521357349
[15]: http://www.xemacs.org
[16]: http://www.mozilla.org
[17]: http://groups.google.com/groups?q=alt.fan.jwz&meta=site%3Dgroups
[18]: http://en.wikipedia.org/wiki/DNA_Lounge
[19]: http://norvig.com#answers
[20]: http://en.wikipedia.org/wiki/No_Silver_Bullet
[21]: http://www.amazon.com/exec/obidos/ASIN/034531509X
[22]: http://citeseer.nj.nec.com/context/7718/0
[23]: http://books.google.com/books?id=dYPSHAAACAAJ&dq=%22perception+in+chess%22+simon&ei=z4PyR5iIAZnmtQPbyLyuDQ
[24]: http://python.org
[25]: http://www.schemers.org
[26]: https://www.khanacademy.org/computing/cs/programming
[27]: http://alice.org
[28]: http://www.squeak.org/
[29]: https://blockly-demo.appspot.com/static/apps/index.html
[30]: http://www.amazon.com/gp/product/0262011530
[31]: http://www.swiss.ai.mit.edu/classes/6.001/abelson-sussman-lectures/
[32]: http://mitpress.mit.edu/sicp/full-text/book/book.html
[33]: http://www.amazon.com/gp/product/0262062186
[34]: http://www.amazon.com/gp/product/1887902996
[35]: http://wiki.python.org/moin/BeginnersGuide
[36]: http://www.amazon.com/gp/product/0262220695
[37]: http://norvig.com/150px-Arabic_Text.png
[38]: http://norvig.com/200px-Flag_of_Bulgaria.svg.png
[39]: http://norvig.com/ch-flag.gif
[40]: http://www.javaresearch.org/article/showarticle.jsp?column=451&thread=12568
[41]: http://norvig.com/hr-flag.gif
[42]: http://tvrtko.bedekovic.net/home/articles/programming/21-days.html
[43]: http://upload.wikimedia.org/wikipedia/commons/thumb/f/f5/Flag_of_Esperanto.svg/200px-Flag_of_Esperanto.svg.png
[44]: http://federicogobbo.name/eo/norvig-21-tagoj.html
[45]: http://norvig.com/fr-flag.gif
[46]: http://www.vizmatic.com/apprenez-a-programmer-en-dix-ans/lang/fr
[47]: http://norvig.com/gm-flag.gif
[48]: http://purl.net/stefan_ram/html/21-tage
[49]: http://norvig.com/heb-150.png
[50]: http://www.reviewsnest.com/charities/programmingin10years.htm
[51]: http://norvig.com/in-flag.gif
[52]: http://vikashtiwari.blogspot.com/2008/04/teach-yourself-programming-in-10-years.html
[53]: http://norvig.com/hu-flag.gif
[54]: http://mestyanm.extra.hu/doc/21_nap.html
[55]: http://norvig.com/id-flag.png
[56]: http://djitz.com/djitzlosophy/belajar-programming-sendiri-dalam-sepuluh-tahun/
[57]: http://norvig.com/it-flag.gif
[58]: http://www.culturahacker.it/documenti/teach.html
[59]: http://norvig.com/ja-flag.gif
[60]: http://www.yamdas.org/column/technique/21-daysj.html
[61]: http://norvig.com/ks-flag.gif
[62]: http://www.tavon.org/site/work/21-days
[63]: http://norvig.com/ir-flag.gif
[64]: http://barnamenevis.org/forum/showthread.php?t=150950
[65]: http://norvig.com/pl-flag.gif
[66]: http://jnowak.students.wmid.amu.edu.pl/norvig.html
[67]: http://norvig.com/br-flag.gif
[68]: http://pihisall.wordpress.com/2007/03/15/aprenda-a-programar-em-dez-anos/
[69]: http://norvig.com/ro-flag.gif
[70]: http://norvig.com/romanian21-days.html
[71]: http://norvig.com/rs-flag.gif
[72]: http://www.williamspublishing.com/21-days.html
[73]: http://norvig.com/rb-flag.gif
[74]: http://www.inverudio.com/21-days.htm
[75]: http://norvig.com/sp-flag.gif
[76]: http://loro.sf.net/notes/21-dias.html
[77]: http://norvig.com/lo-lgflag.gif
[78]: http://www.efton.sk/sk/learn_programming_in_10_years.html
[79]: http://norvig.com/tu-flag.gif
[80]: http://ileriseviye.org/arasayfa.php?inode=programmingtenyears.html
[81]: http://norvig.com/ukranian-flag.png
[82]: http://oim.asu.kpi.ua/2013/03/18/piter-norvih-navchitsya-prohramuvaty-za-desyat-rokiv/
[83]: http://norvig.com/index.html
[84]: http://norvig.disqus.com/?url=ref

