---
created_at: '2017-09-02T17:18:29.000Z'
title: Hell is a multi-threaded C++ program (2006)
url: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html
author: mpweiher
points: 223
story_text: 
comment_text: 
num_comments: 122
story_id: 
story_title: 
story_url: 
parent_id: 
created_at_i: 1504372709
_tags:
- story
- author_mpweiher
- story_15157116
objectID: '15157116'

---
[Source](http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html "Permalink to Another Day In The Code Mines: Hell is a multi-threaded C++ program.")

# Another Day In The Code Mines: Hell is a multi-threaded C++ program.

#  [ Another Day In The Code Mines ][1]

Yet another infrequently-updated blog, this one about the daily excitement of working in the software industry.

## Saturday, September 02, 2006

###  Hell is a multi-threaded C++ program. 

### What are threads?

Every modern operating system has support for threads, and most programming environments provide some level of support for threading. What threads give you is the ability for your program to do more than one thing at once. The problem with threads is the way that they can dramatically increase the complexity of your program.  
  
First, a little background, so we're all on the same page. In Computer Science, as in the physical sciences, using a simplified model makes it easier to discuss complex phenomena without getting bogged down in insignificant details. The trick of course, is in knowing where your simplifications deviate from reality in a way that affects the validity of the results. While spherical cows on an infinite frictionless plane _do_ make the calculations easier, sometimes the details matter.  
  
When **Real Computer Scientists (tm)** are discussing problems in concurrent programming (like the Dining Philosophers), they'll sometimes refer to a _Process_, which is kind of abstract ideal of a computer program. Multiple Processes can be running at the same time in the same system, and can also interact and communicate in various ways.  
  
The threads provided by your favorite operating system and programming language are something basically similar to this theoretical concept of a Process, with a few unfortunate details of implementation.  
  

### The New Jersey approach

I couldn't find a definitive reference to the history of the development of threads as we know them today, but the model most people are familiar with arose out of POSIX, which was largely an attempt to formalize existing practice in UNIX implementations.  
  
It turns out that POSIX Threads, Mach Threads, Windows Threads, Java Threads, and C# Threads all work very much the same, since they're all implemented in more or less the same way. The object-oriented environments wrap a thin veneer of objects around a group of extremely low-level functions, but you've got your basic operations of create(), join(), and exit(), as well as operations on condition variables and mutexes. For the rest of this rant, I'll refer to these as "Pthreads", for convenience.  
  
Pthreads are an example of the [Worse is better][2] philosophy of software design, as applied to the problems of concurrent programming. The POSIX threading model is just about the simplest possible implementation of multi-threading you could have. When you want to create a new thread, you call pthread_create(), and a new thread is created, starting execution with some function you provide. The newly-created thread is created by allocating some memory for a stack for the new thread, loading up a couple of machine registers, and jumping to an address.  
  

### Shared state - two models

In the Pthreads model, all of your threads share the same address space. This makes sharing data between threads very simple and efficient. On the other hand, the fact that all of the state in the program is accessible and changeable from every thread can make it very difficult to ensure that access to all this shared state is managed correctly. Race conditions, where one thread attempts to update a data structure at the same time that another thread is trying to access or change that same structure, are common.  
  
The problem with the _all state is shared_ model is that it doesn't match up very well with what you're generally trying to accomplish when you spawn a thread. You'll normally create a new thread because you want that thread to do something _different_ than what the main thread is already doing. This implies that not all of the state in the parent thread needs to be available to be modified in the second thread. But because of the way threads are created in this model, it's easier (for the OS or language implementor) to share everything rather than a well-defined subset, so that's what you get.  
  
The other major model for multi-threading is known as message-passing multiprocessing. Unless you're familiar with the [Occam][3] or [Erlang][4] programming languages, you might not have encountered this model for concurrency before.  
  
There are a number of variations on the message-passing model, but they all have one thing in common: In the message-passing model, your threads _don't share any state_ by default. If you want some information to go from one thread to another, you need to do it by having one thread send a message to the other thread, typically by calling a function provided by the system for just this purpose. Two popular variants of the message-passing model are ["Communicating Sequential Processes"][5] and the ["Actor model"][6].  
  
You can get a nice introduction to the message-passing model by reading the first couple chapters of the [Occam Reference Manual][7], which is apparently available online these days (I got mine by digging around in a pile of unwanted technical books at a former employer). Occam is of course the native language of the Transputer, a very inventive but commercially unsuccessful parallel processor architecture from the UK which made a big splash in the mid-80's before vanishing without a trace.  
  
Why would you want to learn about this alternative model, when Pthreads have clearly won the battle for the hearts and minds of the programming public? Well, besides the sheer joy of learning something new, you might develop a different way of looking at problems, that'll help you top make better use of the tools that you do use regularly. In addition, as I'll explain in Part II of this rant, there's good reason to believe that message-passing concurrency is going to be coming back in a big way in the near future.  
  

### Enough rope to hang yourself with

As I mentioned earlier, the Pthreads model implies that all of your program's address space is shared between all threads. Most (all?) implementations allow you to allocate some amount of thread-local storage, but in general, the vast majority of your program's state is shared by every thread. This implies that every thread has the ability to modify the value of any variable, and call any arbitrary function, at any time. This is a really powerful tool, but like all powerful tools, it can be dangerous if misused.  
  
It's extremely difficult to predict what the behavior of even fairly simple code will be, when multiple threads can run it simultaneously. For more complex code, the problem rapidly becomes intractable. In a low-level language like C, you need to know the intimate details of how the compiler will optimize your code, which operations are guaranteed to be completed atomically, what the register allocation policy is, etc, etc. In a JIT-compiled language like Java or C#, it's impossible to even know what machine code will be used at runtime, so analyzing runtime behavior in detail just isn't possible.  
  

### An unlocked gun cabinet

I think one of the major problems with Pthreads is that _it's too easy to make something that almost works_. This then leads to an unwarranted belief that multi-threaded programming is simple. For example, say you've got a simple interactive GUI application, and you think that the application takes too long to calculate something after the user presses a button. The "obvious" solution to this problem is to have your button-press handler spawn off a new thread to perform the long-running operation.   
  
So you try this, and it works perfectly on the first try - the child thread launches, and then calculates away while the main thread goes back to handling the UI. You still need to notify the main thread when the calculation is complete, but there any number of easy ways to do this, and you probably won't have much trouble figuring that out. _Gee, that wasn't so difficult, I wonder why people say that multi-threaded programming is difficult?_  
  
It's this sort of ad-hoc approach to creating threads that gets people into trouble. They create a new thread to solve one problem, and then another, and then they suddenly realize that thread A and thread M are interacting in a bad way. So they protect some critical data structures with mutexes, and before they know it, they're trying to debug a deadlock situation where they don't even understand how those two pieces of code could interact.   
  
You need to really think about _why_ you're creating a thread, before you spawn it. I'm not going to go so far as to say that creating threads while your program is running (rather than at startup) is de-facto proof that you're doing something wrong, but it's definitely a strong indication that you're not thinking about _what your threads are for_ with any great rigor.  
  

### How to not shoot yourself in the foot

... is going to be the subject of [Part II][8]. Sorry for the cliff-hanger ending, but I wanted to get at least a little of this published, and potentially get some comments on it, before finishing the rest. 

Posted by  [ Mark Bessey ][9] at  [6:49 PM][10] [ ![][11] ][12] [ ![][13] ][14]

#### 10 comments:

![][15][ ][16]

[leeg][16] said... 
: 

Maybe you could clarify something for me: I think that your second model of threading can be implemented in Mach, with some resource issues; rather than simply spawning a new thread, you need to spawn a new _task_ containing your thread and tell vm_inherit() that you don't want any of the memory regions to be inherited. This effectively gives you a poor man's fork(), so that you get a new task but without it even having a shadow object pointing to the first task's data. It's not as simple as the native Mach model (which is indeed pthread-esque) but I don't think it's mad hard to do.  
  
Does that implement what you're talking about, or are me and the point on different continents? :-) 

:  [ 4:34 AM, September 04, 2006 ][17] [ ![][18] ][19]

![][20]

Anonymous said... 
: 

Great to see some appreciation and awareness of the Occam language!  
  
I hope the ideas embodied in it will see the light of day again.  
  
(Erlang is cool too, but not exactly the same..) 

:  [ 10:15 PM, September 05, 2006 ][21] [ ![][18] ][22]

![][23]

Reimer Mellin said... 
: 

nice writeup, but if memory does not trick me then CSP was indeed the granddaddy of message passing models and predated ERlang and definitly Occam. 

:  [ 10:26 PM, September 05, 2006 ][24] [ ![][18] ][25]

![][26]

Perry The Cynic said... 
: 

It seems to me that most thread programming is done for three reasons:  
(1) Take advantage of multiple CPU cores for a single task.  
(2) Work with (or around) synchronous, blocking interfaces that would otherwise idle the processor.  
(3) Avoid asynchronous/state model programming and continue to use sequential programming techniques instead.  
  
Only (1) is a good reason to use threads of any kind (and then only if you're doing a CPU-crunching job).  
  
(2) is sometimes necessary to keep your program from freezing up, but the right answer is to use (and demand) asynchronous/nonblocking APIs to get your work done. (Sadly, AIO-style UNIX I/O, to give an example, is often implemented with threads, propagating the problem right into the new API.)  
  
And then there's (3), the source (in my estimate) of a good 80-90% of all thread programming. State/event programming seems so unnatural to many programmers that they'll go to incredible lengths to avoid it. One reason is that by the time the problem becomes obvious, programs are already established (often shipped), and rewriting them as state machines is a major architectural change. Adding just one thread (or two, or ten) seems so much easier, a more gradual, manageable change. Many programs slip into multi-threading that way, almost imperceptively. The result is almost invariably mush.  
  
Cheers  
\-- perry 

:  [ 11:22 PM, September 06, 2006 ][27] [ ![][18] ][28]

![][29]

Mike said... 
: 

I really think the "threads are bad" mentality comes from the poor (as in non-existent) support for concurrent programming in C/C++. "Threads" are a poor concurrency model compared to (for example) protected objects and tasks in Ada. I have done concurrent programming in Ada83 and Ada95 for years and the built-in support for concurrency in these languages makes development of multithreaded code much easier, IMO, than using threads, condition variables, mutexes etc in C. Sure, you could argue that on Unix systems Ada tasks are in fact implemented as threads and that rendezvous mechanisms use condition variables and mutexes, but so what? As an Ada programmer, I don't have to deal with all those low-level objects, all of which have different APIs. Instead I can work with the single tasking model API in the language.   
  
Other languages might have concurrent programming support that is nice too (Occam per article), but Ada is the only language I've worked in that had it. I hope Java and Ruby eventually add true concurrent programming support as well and ditch the C/C++ "thread" model. 

:  [ 7:37 AM, September 07, 2006 ][30] [ ![][18] ][31]

![][32][ ][33]

[Anne][33] said... 
: 

Hi, this is a great post and sums up basically how I feel about threads (and a lot of other programming idioms too).  
  
I came here in order to find a blog or message board that makes fun of bad c++ code. Not so much to critique it, but to say "Can you believe someone did this in production code? Ha ha funny!" Do you know of such a place? 

:  [ 1:20 PM, October 20, 2006 ][34] [ ![][18] ][35]

![][20]

Anonymous said... 
: 

""Can you believe someone did this in production code? Ha ha funny!" Do you know of such a place?"  
  
The Daily WTF is your source for that kind of humor:  
http://thedailywtf.com/Series/CodeSOD.aspx 

:  [ 5:09 AM, January 10, 2008 ][36] [ ![][18] ][37]

![][20]

Anonymous said... 
: 

Coming from the JavaScript world and it's way too insulated WebWorker model, I appreciate threads so much more. There you have to send both the source and the data that you wish to run.  
  
  
I have to fault the statement that the threads have access to everything in your application. Yes address space wise they do, and if you have a lot of singletons/globals on the threaded path, then you have trouble. But if you build and insulate the data well, then it's not as awedul as you would have readers believe.  
  
Libdispatch atop a system thread pool balanced across all running apps is a really great threading experience. And threads aren't that bad if you name and track what they do.  

:  [ 7:30 PM, September 02, 2017 ][38] [ ![][18] ][39]

![][20]

Anonymous said... 
: 

_you want that thread to do something different than what the main thread is already doing..._  
  
Maybe isolate the two things into separate programs/processes and have them interact using shared memory?  
  
This is just as fast as the thread model but you only share exactly the state needed for the two to communicate.  

:  [ 11:48 PM, September 04, 2017 ][40] [ ![][18] ][41]

![][42][ ![][43]][44]

[Mark Bessey][44] said... 
: 

The communication part is just as fast, but creating a thread is much faster than creating a process. But yes - this is definitely a way to do it. 

:  [ 11:23 AM, September 06, 2017 ][45] [ ![][18] ][46]

[Post a Comment][47]

#### [][48][

[Newer Post][49] [Older Post][50] [Home][1]

Subscribe to: [Post Comments (Atom)][51]

## About Me

![My photo][52]

[ Mark Bessey ][44]
: Santa Barbara, California, United States

[View my complete profile][44]

[ ![][53] ][54]

## Twitter

[Follow @mbessey][55]

[ ![][53] ][56]

## Get updates via email

| ----- |
|   |   | 

[ ![][53] ][57]

| -----|  
|  

  |  

  |  

## Blog Archive

* [ ►  ][58] [ 2017 ][59] (11)
    * [ ►  ][58] [ November ][60] (3)
    * [ ►  ][58] [ October ][61] (2)
    * [ ►  ][58] [ September ][62] (3)
    * [ ►  ][58] [ August ][63] (3)
* [ ►  ][58] [ 2016 ][64] (2)
    * [ ►  ][58] [ December ][65] (1)
    * [ ►  ][58] [ February ][66] (1)
* [ ►  ][58] [ 2014 ][67] (1)
    * [ ►  ][58] [ September ][68] (1)
* [ ►  ][58] [ 2013 ][69] (3)
    * [ ►  ][58] [ February ][70] (1)
    * [ ►  ][58] [ January ][71] (2)
* [ ►  ][58] [ 2012 ][72] (2)
    * [ ►  ][58] [ February ][73] (2)
* [ ►  ][58] [ 2011 ][74] (1)
    * [ ►  ][58] [ June ][75] (1)
* [ ►  ][58] [ 2010 ][76] (1)
    * [ ►  ][58] [ January ][77] (1)
* [ ►  ][58] [ 2009 ][78] (10)
    * [ ►  ][58] [ June ][79] (1)
    * [ ►  ][58] [ May ][80] (1)
    * [ ►  ][58] [ April ][81] (1)
    * [ ►  ][58] [ March ][82] (2)
    * [ ►  ][58] [ February ][83] (4)
    * [ ►  ][58] [ January ][84] (1)
* [ ►  ][58] [ 2008 ][85] (25)
    * [ ►  ][58] [ December ][86] (1)
    * [ ►  ][58] [ November ][87] (3)
    * [ ►  ][58] [ October ][88] (1)
    * [ ►  ][58] [ September ][89] (1)
    * [ ►  ][58] [ August ][90] (4)
    * [ ►  ][58] [ July ][91] (1)
    * [ ►  ][58] [ June ][92] (3)
    * [ ►  ][58] [ May ][93] (2)
    * [ ►  ][58] [ April ][94] (1)
    * [ ►  ][58] [ March ][95] (4)
    * [ ►  ][58] [ February ][96] (2)
    * [ ►  ][58] [ January ][97] (2)
* [ ►  ][58] [ 2007 ][98] (14)
    * [ ►  ][58] [ December ][99] (1)
    * [ ►  ][58] [ November ][100] (1)
    * [ ►  ][58] [ October ][101] (2)
    * [ ►  ][58] [ May ][102] (3)
    * [ ►  ][58] [ April ][103] (1)
    * [ ►  ][58] [ March ][104] (1)
    * [ ►  ][58] [ February ][105] (2)
    * [ ►  ][58] [ January ][106] (3)
* [ ▼  ][58] [ 2006 ][107] (12)
    * [ ►  ][58] [ November ][108] (1)
    * [ ▼  ][58] [ September ][109] (3)
        * [A couple of quick links...][110]
        * [Another thread on . . . threads][8]
        * [Hell is a multi-threaded C++ program.][111]
    * [ ►  ][58] [ August ][112] (3)
    * [ ►  ][58] [ July ][113] (2)
    * [ ►  ][58] [ June ][114] (1)
    * [ ►  ][58] [ January ][115] (2)
* [ ►  ][58] [ 2005 ][116] (3)
    * [ ►  ][58] [ October ][117] (1)
    * [ ►  ][58] [ August ][118] (1)
    * [ ►  ][58] [ June ][119] (1)

[ ![][53] ][120]

| -----|  
|  

  |  

  |  

Simple theme. Powered by [Blogger][121]. 

[ ![][53] ][122]

[1]: http://codemines.blogspot.com/
[2]: http://www.jwz.org/doc/worse-is-better.html
[3]: http://en.wikipedia.org/wiki/Occam_programming_language
[4]: http://en.wikipedia.org/wiki/Erlang_programming_language
[5]: http://en.wikipedia.org/wiki/Communicating_sequential_processes
[6]: http://en.wikipedia.org/wiki/Actor_model
[7]: http://wotug.org/occam/documentation/oc21refman.pdf
[8]: http://codemines.blogspot.com/2006/09/another-thread-on-threads.html
[9]: https://www.blogger.com/profile/12091448340989293403 "author profile"
[10]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html "permanent link"
[11]: https://resources.blogblog.com/img/icon18_email.gif
[12]: https://www.blogger.com/email-post.g?blogID=14034177&postID=115536076404339034 "Email Post"
[13]: https://resources.blogblog.com/img/icon18_edit_allbkg.gif
[14]: https://www.blogger.com/post-edit.g?blogID=14034177&postID=115536076404339034&from=pencil "Edit Post"
[15]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "leeg"
[16]: https://www.blogger.com/profile/07305141119009757571
[17]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1157369640000#c115736969118298566 "comment permalink"
[18]: https://resources.blogblog.com/img/icon_delete13.gif
[19]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=115736969118298566 "Delete Comment"
[20]: http://img1.blogblog.com/img/blank.gif "Anonymous"
[21]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1157519700000#c115751971151979959 "comment permalink"
[22]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=115751971151979959 "Delete Comment"
[23]: http://img1.blogblog.com/img/blank.gif "Reimer Mellin"
[24]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1157520360000#c115752037964214355 "comment permalink"
[25]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=115752037964214355 "Delete Comment"
[26]: http://img1.blogblog.com/img/blank.gif "Perry The Cynic"
[27]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1157610120000#c115761012714317884 "comment permalink"
[28]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=115761012714317884 "Delete Comment"
[29]: http://img1.blogblog.com/img/blank.gif "Mike"
[30]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1157639820000#c115763985538847562 "comment permalink"
[31]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=115763985538847562 "Delete Comment"
[32]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Anne"
[33]: https://www.blogger.com/profile/06225074649035089920
[34]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1161375600000#c116137564876264914 "comment permalink"
[35]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=116137564876264914 "Delete Comment"
[36]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1199970540000#c5233383795894812140 "comment permalink"
[37]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=5233383795894812140 "Delete Comment"
[38]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1504405846772#c830113891221606501 "comment permalink"
[39]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=830113891221606501 "Delete Comment"
[40]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1504594080970#c2454422019313880805 "comment permalink"
[41]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=2454422019313880805 "Delete Comment"
[42]: https://img1.blogblog.com/img/blank.gif "Mark Bessey"
[43]: http://3.bp.blogspot.com/_MR7bT7RHlnY/SjqZh5hjsRI/AAAAAAAAAHo/fErMHtziYLk/S45-s35/n1434932199_1738.jpg
[44]: https://www.blogger.com/profile/12091448340989293403
[45]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html?showComment=1504722214927#c6170588183718720020 "comment permalink"
[46]: https://www.blogger.com/delete-comment.g?blogID=14034177&postID=6170588183718720020 "Delete Comment"
[47]: https://www.blogger.com/comment.g?blogID=14034177&postID=115536076404339034
[48]: 
[49]: http://codemines.blogspot.com/2006/09/another-thread-on-threads.html "Newer Post"
[50]: http://codemines.blogspot.com/2006/08/cockatoo-joke.html "Older Post"
[51]: http://codemines.blogspot.com/feeds/115536076404339034/comments/default
[52]: http://3.bp.blogspot.com/_MR7bT7RHlnY/SjqZgPOy0GI/AAAAAAAAAHQ/obghMBMSYPo/S220-s48/n1434932199_1738.jpg
[53]: https://resources.blogblog.com/img/icon18_wrench_allbkg.png
[54]: //www.blogger.com/rearrange?blogID=14034177&widgetType=Profile&widgetId=Profile1&action=editWidget&sectionId=sidebar-right-1 "Edit"
[55]: https://twitter.com/mbessey
[56]: //www.blogger.com/rearrange?blogID=14034177&widgetType=HTML&widgetId=HTML1&action=editWidget&sectionId=sidebar-right-1 "Edit"
[57]: //www.blogger.com/rearrange?blogID=14034177&widgetType=FollowByEmail&widgetId=FollowByEmail1&action=editWidget&sectionId=sidebar-right-1 "Edit"
[58]: javascript:void(0)
[59]: http://codemines.blogspot.com/2017/
[60]: http://codemines.blogspot.com/2017/11/
[61]: http://codemines.blogspot.com/2017/10/
[62]: http://codemines.blogspot.com/2017/09/
[63]: http://codemines.blogspot.com/2017/08/
[64]: http://codemines.blogspot.com/2016/
[65]: http://codemines.blogspot.com/2016/12/
[66]: http://codemines.blogspot.com/2016/02/
[67]: http://codemines.blogspot.com/2014/
[68]: http://codemines.blogspot.com/2014/09/
[69]: http://codemines.blogspot.com/2013/
[70]: http://codemines.blogspot.com/2013/02/
[71]: http://codemines.blogspot.com/2013/01/
[72]: http://codemines.blogspot.com/2012/
[73]: http://codemines.blogspot.com/2012/02/
[74]: http://codemines.blogspot.com/2011/
[75]: http://codemines.blogspot.com/2011/06/
[76]: http://codemines.blogspot.com/2010/
[77]: http://codemines.blogspot.com/2010/01/
[78]: http://codemines.blogspot.com/2009/
[79]: http://codemines.blogspot.com/2009/06/
[80]: http://codemines.blogspot.com/2009/05/
[81]: http://codemines.blogspot.com/2009/04/
[82]: http://codemines.blogspot.com/2009/03/
[83]: http://codemines.blogspot.com/2009/02/
[84]: http://codemines.blogspot.com/2009/01/
[85]: http://codemines.blogspot.com/2008/
[86]: http://codemines.blogspot.com/2008/12/
[87]: http://codemines.blogspot.com/2008/11/
[88]: http://codemines.blogspot.com/2008/10/
[89]: http://codemines.blogspot.com/2008/09/
[90]: http://codemines.blogspot.com/2008/08/
[91]: http://codemines.blogspot.com/2008/07/
[92]: http://codemines.blogspot.com/2008/06/
[93]: http://codemines.blogspot.com/2008/05/
[94]: http://codemines.blogspot.com/2008/04/
[95]: http://codemines.blogspot.com/2008/03/
[96]: http://codemines.blogspot.com/2008/02/
[97]: http://codemines.blogspot.com/2008/01/
[98]: http://codemines.blogspot.com/2007/
[99]: http://codemines.blogspot.com/2007/12/
[100]: http://codemines.blogspot.com/2007/11/
[101]: http://codemines.blogspot.com/2007/10/
[102]: http://codemines.blogspot.com/2007/05/
[103]: http://codemines.blogspot.com/2007/04/
[104]: http://codemines.blogspot.com/2007/03/
[105]: http://codemines.blogspot.com/2007/02/
[106]: http://codemines.blogspot.com/2007/01/
[107]: http://codemines.blogspot.com/2006/
[108]: http://codemines.blogspot.com/2006/11/
[109]: http://codemines.blogspot.com/2006/09/
[110]: http://codemines.blogspot.com/2006/09/couple-of-quick-links.html
[111]: http://codemines.blogspot.com/2006/09/hell-is-multi-threaded-c-program.html
[112]: http://codemines.blogspot.com/2006/08/
[113]: http://codemines.blogspot.com/2006/07/
[114]: http://codemines.blogspot.com/2006/06/
[115]: http://codemines.blogspot.com/2006/01/
[116]: http://codemines.blogspot.com/2005/
[117]: http://codemines.blogspot.com/2005/10/
[118]: http://codemines.blogspot.com/2005/08/
[119]: http://codemines.blogspot.com/2005/06/
[120]: //www.blogger.com/rearrange?blogID=14034177&widgetType=BlogArchive&widgetId=BlogArchive1&action=editWidget&sectionId=sidebar-right-3 "Edit"
[121]: https://www.blogger.com
[122]: //www.blogger.com/rearrange?blogID=14034177&widgetType=Attribution&widgetId=Attribution1&action=editWidget&sectionId=footer-3 "Edit"

  [*[6:49 PM]: 2006-09-02T18:49:00-07:00

