---
created_at: '2016-02-07T18:43:07.000Z'
title: Amplifying C (2010)
url: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html
author: jhack
points: 140
story_text: 
comment_text: 
num_comments: 101
story_id: 
story_title: 
story_url: 
parent_id: 
created_at_i: 1454870587
_tags:
- story
- author_jhack
- story_11054089
objectID: '11054089'

---
[Source](http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html "Permalink to voodoo slide: Amplifying C")

# voodoo slide: Amplifying C

#  [ voodoo slide ][1]

Nu åker vi

## Friday, January 22, 2010

###  Amplifying C 

## On C++

Having programmed in C++ professionally for well over 10 years I have learned all of it. I have all the books, I know all the tricks. And I don’t like it anymore.

_Update: This intro apparently made some people see red, because "no man could possibly know all of C++". If that includes you, you can read it as: "I've shipped 4 AAA games on MLOC code bases, and here's my take on the C++ abstractions you can reasonably use in projects that big". _

Basically game teams using C++ fall into the same trap every time: they try to create abstractions with whatever is in the C++ toolbox and they fail miserably. On the next project they’re a little bit smarter from the experience so they set out to fix their abstractions and create new ones. And fail.

Quickly going over the major abstraction mechanisms C++ introduced over C I’m arguing that:
* Templates suck as they cause link-time spam and compile times to skyrocket. They severely bloat the size of the debug symbol file, which on large projects can easily reach several hundred megabytes of data. Abstractions built with templates perform differently depending on whether compiler optimizations are enabled or not (every tried debugging Boost code?). They’re essentially unusable on large code bases beyond container-of-T and simple functions. 
* RTTI sucks because it doesn’t do what anyone wants, and you can’t even rely on it returning a type name formatted in a certain way. 
* Classes suck because their guts have to be in headers for all to see. 

All these high-level concepts are flawed and you can’t alter their semantics because they’re set in ISO stone. What all teams do then is to reinvent all the language components that don’t work for them, and sets up rules forbidding the use of the other features. Every C++ shop has its own "accepted" subset.

Basically the C++ game developer community is slowly navigating away from C++'s abstraction patterns. We left operator overloading mostly in the 90’s (some vector libraries still use it). We ditched RTTI back in 2001. Exceptions are firmly off as they don’t even work on all platforms we develop for. A lot of people are advocating that we stop using member functions to reduce coupling.

This may sound harsh, but to me these are clear signs that C++ isn’t providing any real cost benefit for us, and that we should be writing code in other ways.

## Coding C

Many C++ game programmers have started to turn towards C (or C-like C++) to get away from the flaws of C++. Targeting C manually with larger systems can be a lot of work, because it offers very basic abstraction facilities. There are functions, enums, tagged types (structs and unions) and a rudimentary type system, but that’s about it.

But let’s look at C as a platform for a minute. C is lean, compiles super fast, it’s supported everywhere and all the tools we need to ship games such as compilers and debuggers (including the obscure and proprietary) work well with it. If you need platform-specific intrinsics to get on with your job, you can rely on the target platform’s C compiler to provide them.

As C is such a simple, predictable language that works everywhere it makes a lot of sense to generate C code. Indeed many projects have done so, but typically the meat of the application has still been written in plain C as code generation is typically used for language interfaces or parser generators.

## The Lisp Way

Having also programmed a lot of Common Lisp over the years, I’ve seen how the Lisp family of languages deals with extensibility. In Lisp, you write your own abstractions that become a part of the project’s language. This remarkable feature is enabled basically through two simple things:

* Programs can be treated as data (because they can be thought of as parse trees) 
* There are macros which transform such data (that is, your programs) into other programs (implementing the abstractions). 

I’m going to suggest something mildly radical: we should prefer C over C++. But not straight-up C. We should create our own C with the abstractions we need, built right into the language, customized for the problems we’re working on.

## Amplification

We can apply many of the ideas that make Lisp powerful to C if we drop its Algol-like syntax. What is the difference between the following two program fragments?

| ----- |
| 
    
    
    int my_function(int a, int b) {
        return a + b;
    }

 | 

| ----- |
| 
    
    
    (defun my-function ((a int) (b int) (return int))
      (return (+ a b))

 | 

Answer: none, they are equivalent as far as semantics go. The latter can trivially be parsed (using very simple rules) and transformed into the former, and so it is still the same C program. This is good news, because Lisp has shown us that if we represent programs as data, we can transform that data arbitrarily before evaluating or compiling it.

In my prototype system, c-amplify, I’m doing exactly this. The system introduces an "amplification" phase where s-expressions are transformed to C code before a traditional build system runs.

The c-amplify system has the following major parts:

* A system definition facility (specifying input files and dependencies) 
* A reader, parsing ca source files 
* A persistent function and type database which is updated as source files are amplified. 
* A pretty-printing C code generator — important as we’re going to be debugging the generated code. 

The system is intended to be run incrementally as source files are changed, re-reading changed files, updating the database and writing generated output. A traditional build system can then be used to the resulting files.

The persistent database is an interesting component that isn’t strictly needed for the system but enables a lot of neat features:

* Type inference. Because all functions and types are known, c-amplify can easily supply a type-of operator for arbitrary expressions. This can be used as the basis for type inferring macros similar to auto in C++0x or var in C#. 
* Hook functions could be installed that run over the database and do additional work. For example, instrumenting all writes to a particular struct field, generating reflection info or performing project-specific checks on how types or functions are used. The possibilities are pretty much endless. Remember all those times you’ve thought: if we could only access this thing in the compiler we could give an error message if the code does this thing? Well, with hooks you could. 

## RAII, uncluttered

Let’s look at one such macro that solves a real problem: making sure a file handle is closed in an orderly manner, even when there are multiple exit points from the block.

In situations like this, C++ fans can’t wait to tell you about RAII. RAII means creating a stack object of some utility type that performs resource cleanup in its destructor. If we look at the `AutoFile` type we need to type up to implement RAII we find that exactly one line is providing the abstraction we need (the destructor), the rest is boilerplate:

| ----- |
| 
    
    
    class AutoFile // auxillary type
    {
      private:
        FILE* f;
    
      public:
        AutoFile(const char* fn, const char* mode) {
          f = fopen(fn, mode);
        }
        ~AutoFile() { if (f) fclose(f); }
        operator FILE*() { return f; }
    
      private:
        AutoFile(const AutoFile&);
        AutoFile& operator=(const AutoFile&);
    };
    
    // later, that same day..
    {
       AutoFile file("c:/temp/foo.txt", "w");
       fprintf(file, "Hello, world");
    }

 | 

What Lisp programmers do to manage resources is to create block-wrapping macros (usually starting with the word `with-`). The macros sits around a body of code, indicating visually that the code wrapped by the macro will have access to some resource. The macro expansion is guaranteed to clean up the resource regardless of how the block terminates. Here’s an example of using such a macro with c-amplify:

| ----- |
| 
    
    
    (with-open-file (f "c:/temp/foo.txt" "w")
      (fprintf f "Hello, world"))

 | 

If we ask c-amplify to macro-expand this we see that the details of calling `fopen` and `fclose` are being handled as if we had written out everything by hand:

| ----- |
| 
    
    
    {
      FILE* f = (FILE *) 0;
      {
        f = fopen("c:/temp/foo.txt", "w");
        fprintf(f, "Hello, world");
      }
    cleanup_8_:
      if (file) {
        fclose(file);
      }
    }

 | 

Even if we add more complex code with multiple return paths, c-amplify doesn’t let us down:

| ----- |
| 
    
    
    (defun foo ((return int))
      (with-open-file (f "c:/temp/foo.txt" "w")
        (if (> (rand 10) 5)
           (return 20))
        (fprintf f "Hello, world")
        (return 10)))

 | 

This amplifies to the following C code. Note how the with-open-file block locally redefines what it means to return a value. This C code is a close representation of what a C++ compiler has to emit when RAII is used but as before there are no residual types left.

| ----- |
| 
    
    
    int foo(void)
    {
      FILE* f = (FILE *) 0;
      {
        int result_13_;
        {
          f = fopen("c:/temp/foo.txt", "w");
          if (rand(10) > 5) {
            result_13_ = 20;
            goto cleanup_12_;
          }
          fprintf(f, "Hello, world");
          {
            result_13_ = 10;
            goto cleanup_12_;
          }
        }
        cleanup_12_:
        if (f) {
          fclose(f);
        }
        return result_13_;
      }
    }

 | 

One possible c-amplify implementation of the `with-open-file` macro looks like this (on a real game project it would of course not use `fopen`, but some custom file manager):

| ----- |
| 
    
    
    (def-c-macro with-open-file ((var file-name mode) &body body)
      `(progn
         (declare (,var = (cast (ptr #$FILE) 0)))
         (unwind-protect
              (progn
                (= ,var (#$fopen ,file-name ,mode))
                ,@body)
           (when ,var
             (#$fclose ,var)))))

 | 

The funny `#$foo` syntax is just a reader macro to facilitate reading case sensitive symbols in a special package which corresponds to the C namespace. The implementation piggy-backs on `unwind-protect`, which makes sure that the body code always goes through the cleanup clauses:

| ----- |
| 
    
    
    (def-c-macro unwind-protect (form &body cleanup-forms)
      (with-c-gensyms (cleanup result)
        `(progn
           (ast-stmt-if (not (current-defun-void-p))
                        (declare (,result *current-return-type*)))
           (macrolet (return (&optional expr)
                             `(progn
                                (ast-stmt
                                 (if (not (current-defun-void-p))
                                     `(= ,',',result ,,expr)
                                     `(cast void ,,expr)))
                                (goto ,',cleanup)))
             ,form)
           (label ,cleanup)
           ,@cleanup-forms
           (return ,result))))

 | 

If we decided to add exception handling (through e.g. `setjmp` or SEH exceptions), we only have to touch `unwind-protect` to enable exception cleanup in all our RAII-like resource macros. Layering pure compile-time abstractions like this to create programs is incredibly powerful.

## Exploiting the database

As I mentioned, there are many advantages to having all your code parsed with type information sitting around in an in-memory database. Let’s highlight one such thing, automatic type inference for local variables. Consider the following c-amplify input:

| ----- |
| 
    
    
    (defstruct bar
      (x (const restrict volatile ptr int)))
    
    (defstruct foo
      (barp (ptr struct bar)))
    
    (defun my-function ((foop (ptr struct foo)) (return int))
      (let ((xp (-> foop barp x)))
         (return (* xp))))

 | 

This amplifies to:

| ----- |
| 
    
    
    struct bar { int * const volatile restrict x; };
    
    struct foo { struct bar * barp; };
    
    int my_function(struct foo * foop) {
      int * const volatile restrict xp = foop->barp->x;
      return *xp;
    }

 | 

We can see that the lexical variable `xp` has the expected type. The c-amplify system knows how to compute the type of any C expression (including arithmetic promotion) so the this feature can be used extensively if desired.

## Including what’s needed

Another great feature of having a complete function/type database is that generated C files do not need include statements. If a source file needs a bunch of declarations the generator will just emit them right there. There’s no need to maintain header files. If code in a generated file starts using a structure all of a sudden, a copy of its declaration will automatically pop in to the generated file.

For a full-out implementation of this idea to work, 3rd party declarations from the OS and C libraries must be imported into the amplification database. A separate tool must be devised for this but it would certainly be possible.

Compiling files generated like this would mean the the preprocessor wouldn’t touch disk except to read the input c file. Makefiles for generated files such as these will also be trivial to write as there are no implicit dependencies.

## Further ideas

Here are additional ideas that could be explored within the c-amplify system:

* Improved compile-time checking for traditionally dangerous functions (`scanf`, `printf`) (make macros that evaluate the format strings and types of arguments) 
* Add exception handling to C on top of `setjmp`, SEH or some other basic mechanism 
* Annotate structures for real-time tweaking. 
* Generate script language bindings at compile time via macros and hooks. 
* Inlining/code simplification at amplification time (trig function simplification, maths) 
* Add a proper sublanguage for vector math. Finally you can write that vector math library that combines plus and multiply to `madd` on altivec by analyzing the code at compile time, and you don’t need 3000 lines of C++ "expression templates" to do it 
* If you absolutely must have C++-like classes and templates, you could implement those too. Classes with single dispatch would be pretty easy (generate a couple of structs per class), and templates could be "done right" in the sense that you’d only generate a single expansion for each instantiated type and dump them all to a single source file, rather than compiling thousands of instantiations of `std::vector` and letting the linker sort through the carnage. 

## Conclusion

If there is a way (no matter how much work it would be) to express the semantics of an abstraction in C, chances are you can implement it as a set of macros and hooks in c-amplify.

However, c-amplify is still a prototype and a lot of work remains before it might be suitable for production use. I hope this rant has given you some new ideas on how we design programs. Send your feedback and flames my way.

Posted by  [ dep ][2] at  [5:26 PM][3] [ ![][4] ][5]

Labels: [c++][6], [lisp][7], [rant][8]

#### 38 comments:

![][9][ ![][10]][11]

[repi][11] said... 
: 

Great post! Look forward to hearing more about it both here and at work :) 

:  [ 7:17 PM ][12] [ ![][13] ][14]

![][15][ ![][16]][17]

[Jonas][17] said... 
: 

Goose bumps :) 

:  [ 7:55 PM ][18] [ ![][13] ][19]

![][20][ ][21]

[p10q][21] said... 
: 

That sounds really awesome. I'd recommend you integrate something like ccache into to your system as well (so that compile times are very fast).  
  
Also, from a debugging standpoint you might think about how to link (maybe just by how the c code is named) the c-amplify lisp code and the generated c code. Ideally, it'd be awesome to step through both code at the same time...  
  
That persistent database is a really good idea though. Like you say, opens up so many possibilities.   
  
I noticed you mention gvim on this blog. With that database, you could treat the c-amplify code as basically 'folded' c code (accessing it via a client to the persistent database). That would be pretty awesome...  
  
Anyway, cheers. 

:  [ 9:03 PM ][22] [ ![][13] ][23]

![][24]

EvanM said... 
: 

This is some extremely interesting work. I am an experienced C++ programmer, and I share your pains, but some of the things you can do with templates are just too awesome to pass up. I've been loving boost::variant these days, but dear God has it wrecked my compile times! It seems like something equivalent could be implemented with c-amplify relatively painlessly, and without the massive compile times associated with C++ templates. Plus, the automatic header generation would be just delightful.  
  
Any chance that you'll ever open-source your prototype? 

:  [ 1:51 AM ][25] [ ![][13] ][26]

![][27][ ][28]

[David Reynolds][28] said... 
: 

This is really cool. I've been doing a lot of work in C lately and I'm not really a fan of C++. Working in a language like lisp to generate C code is great. I'm looking forward to seeing where you take this. 

:  [ 3:49 AM ][29] [ ![][13] ][30]

![][31]

Anonymous said... 
: 

How is this different than bigloo?  
  
http://www.bigloo.org 

:  [ 3:50 AM ][32] [ ![][13] ][33]

![][34][ ][35]

[malkia][35] said... 
: 

fellow game developer here...  
  
Have you ever used PowerLoom?  
  
To implement PowerLoom we developed a new programming language called STELLA, which is a Strongly Typed, Lisp-like LAnguage that can be translated into Lisp, C++ and Java. PowerLoom is written in STELLA and therefore available in Common-Lisp, C++ and Java versions. 

:  [ 4:05 AM ][36] [ ![][13] ][37]

![][38][ ][39]

[Christopher Armstrong][39] said... 
: 

I know people who have prototyped pretty much identical systems and I've worked on one myself. One of the ones I know already had the exceptions implemented with setjmp/longjmp. Though your type inference thing is impressive.  
  
But we really need someone to release and maintain a high-quality implementation of the idea! (hint hint) :) 

:  [ 4:46 AM ][40] [ ![][13] ][41]

![][42]

Colin Curtin said... 
: 

typo: for for  
  
This is great, and I'm looking forward to how you go about this. This would not only be awesome for games. 

:  [ 5:11 AM ][43] [ ![][13] ][44]

![][45]

Scott said... 
: 

Oracle's in-database Java VM implementation is based on exactly this idea. Lisp is a Gatling Gun for writing C. 

:  [ 5:24 AM ][46] [ ![][13] ][47]

![][48][ ][49]

[WebDevHobo][49] said... 
: 

It is a sad thing that people can't get past their love for C++ and see that there is such a thing as a "paradigm".  
  
Every problem has a best way to get to the answer. And that's not always included in the same programming language.  
  
But... y'know... IT'S C++ IT CAN DO ANYTHING AND WINDOWS SUCKS!!!  
the argument usually goes. 

:  [ 9:28 AM ][50] [ ![][13] ][51]

![][52][ ][53]

[dep][53] said... 
: 

Thanks for all the comments.  
  
@Anonymous: This is different from Bigloo scheme in that it is a way to write fully typed C code, rather than a way to translate untyped Scheme to C. I think of it a translation device with macro capabilities rather than a compiler.  
  
@malkia: I hadn't heard about PowerLoom and STELLA before, I'll have to read up! 

:  [ 10:01 AM ][54] [ ![][13] ][55]

![][56][ ![][57]][58]

[Jyaif][58] said... 
: 

"stop using member functions to reduce coupling"   
What what what? 

:  [ 2:07 PM ][59] [ ![][13] ][60]

![][61][ ][62]

[Ali Sabil][62] said... 
: 

Couldn't agree more, I gave up on c++ few years ago, and I did it for the same reasons you are outlining in your post. Nowadays I program in Vala, which is in my opinion what c++ should have been. 

:  [ 5:17 PM ][63] [ ![][13] ][64]

![][65][ ][66]

[patrickdlogan][66] said... 
: 

You might want to look at Gambit C-Scheme. There are people writing games in it as well, including for the iPhone. It can get fairly compact: iphone, also someone ported it to the Nintendo DS handheld. 

:  [ 5:30 PM ][67] [ ![][13] ][68]

![][69][ ][70]

[eryx][70] said... 
: 

Look at [SC][71]  
This is expandable C in lisp with lisp syntax where you can use lisp style   
macroses and define your own constructs.  
  
It help me a lot in generating my C projects.  
  
Example of function in SC:  
  
(def (fib n) (fn int int)  
(if (<= n 2)   
(return 1)  
(return (+ (fib (- n 2))  
(fib (- n 1))))))  
  
=>  
  
int fib (int n)  
{  
if (n <= 2)  
return 1;  
else  
return fib (n - 2) + fib (n - 1);  
}  
  
Structure declaration in SC:  
  
((struct list)  
(data int)  
(next (ptr (struct list))))  
  
Example of macro in SC:  
  
(%defmacro ntimes (n &body form)  
`(for ((def i int 0) (< i ,n) (inc i)) ,@form)) 

:  [ 8:11 PM ][72] [ ![][13] ][73]

![][74][ ][75]

[dimatura][75] said... 
: 

Check out Lush http://lush.sourceforge.net 

:  [ 8:14 PM ][76] [ ![][13] ][77]

![][31]

Anonymous said... 
: 

Or you could use:  
  
shared_ptr f(fopen(fn,mode),fclose);  
  
From tr1 or boost. 

:  [ 12:19 AM ][78] [ ![][13] ][79]

![][80][ ][81]

[Maybe Attending][81] said... 
:  This comment has been removed by the author.

:  [ 12:56 AM ][82] [ ![][13] ][83]

![][84][ ][81]

[Johan Kotlinski][81] said... 
: 

Tried to post yesterday, but guess it fell out. Anyway one more try:  
  
"Classes suck because their guts have to be in headers for all to see."  
  
How do you think Lisp is an improvement over C++ in this regard? In my experience Lisp is more like "everyone sees everything," no real support for information hiding besides wrapping stuff inside functions. Or do you think more of compile and link times? 

:  [ 12:58 AM ][85] [ ![][13] ][86]

![][69][ ][70]

[eryx][70] said... 
: 

> Johan Kotlinski said...  
> How do you think Lisp is an improvement   
> over C++ in this regard?   
> In my experience Lisp is more like   
> "everyone sees everything,"   
> no real support for information hiding  
> besides wrapping stuff inside functions.  
  
You can use packages, they have protected(internal) and public(exported) symbols.  
  
For CLOS classes and generics with 50  
lines of MOP you can add private slots/methods, "friend" slots/methods   
or any other sort of protection you like.  
  
But nobody really need it as I can see. 

:  [ 5:13 AM ][87] [ ![][13] ][88]

![][89][ ][90]

[temoto][90] said... 
: 

Great job! The most important part of your rant is constructive one: "C++ is flawed *and i propose this solution...*".  
  
What is especially interesting about your project is types database. On the first look, it looks very promising. But it is unclear how are you going to deal with multiple same names. Are you going to introduce some kind of "import" statement that would remove ambiguity on which structure to use? 

:  [ 1:36 PM ][91] [ ![][13] ][92]

![][93]

NONAME said... 
: 

"...I have learned all of it. I have all the books, I know all the tricks"  
  
From this sentence, I stopped taking this article too seriously. 

:  [ 10:01 PM ][94] [ ![][13] ][95]

![][96][ ][97]

[Jason Messer][97] said... 
: 

I generally try to pretend that the problem parts of C++ don't exist, but I still love the STL. Used carefully it reduces or eliminates an awful lot of plumbing code.  
  
Any plans to try and salvage the STL for your project? 

:  [ 7:46 AM ][98] [ ![][13] ][99]

![][100][ ][101]

[rampelstinskin][101] said... 
: 

Drop Algol-like syntax not necessary.  
http://nemerle.org/ 

:  [ 11:48 AM ][102] [ ![][13] ][103]

![][104][ ][105]

[Dimitris Menounos][105] said... 
: 

I think you would love [Vala][106]! It aligns perfectly with your goal of sanely enhancing C with higher level abstractions while staying 99% syntax familiar. 

:  [ 2:34 PM ][107] [ ![][13] ][108]

![][31]

Anonymous said... 
: 

Keep posting stuff like this i really like it 

:  [ 9:25 AM ][109] [ ![][13] ][110]

![][111][ ][112]

[Larry Clapp][112] said... 
: 

Anecdote re: "knowing all of C++": A friend of mine, who is very smart and very experienced and does *not* claim to know all of C++ once interviewed someone that did.  
  
Interviewer 1, to Candidate (on phone): So, Candidate, how much C++ do you know?  
Candidate: All of it.  
Interviewer 1: [_Taken aback]_ Really.  
Candidate: Yes. Ask me anything.  
Interviewer 1: Hold on a minute. _Hey Chris, can you come in here, and bring your ARM?_  
  
They ask random questions from the Annotated C++ Reference Manual, and Candidate does indeed get them all right, and does indeed seem to have all this knowledge at his mental fingertips.  
  
They hire the guy, and he is very smart and very productive and indeed knows all of C++.  
  
I later worked with him, and asked him about it, and he said that that knowledge had atrophied and he no longer claimed to know all of C++.  
  
So it can happen. :)  
  
Re the article: I like this idea, and along parallel lines have often wondered what Perl would look like in Lisp syntax.  
  
One issue I see is that in situations like this you often end up having to know *both* languages (in this case, CL and C) to get a lot done. I see signs of this in your post, e.g. the "with-open-file" macro. This is "obvious" to a CL'er but not so much to a C or C++ programmer. Beware of undocumented crossover idioms. :) 

:  [ 2:39 PM ][113] [ ![][13] ][114]

![][111][ ][112]

[Larry Clapp][112] said... 
: 

Also: ECL is a Common Lisp compiler that generates C code. Just a thought. :) 

:  [ 2:41 PM ][115] [ ![][13] ][116]

![][117][ ][118]

[cl-beer][118] said... 
: 

Thank you, I have been trying to articulate this for years. I dislike C++ and I have a few good, but now I can add yours as well. I love lisp (and it's little brother, little scheme) and think it's a perfect match for game development. Oh, and keep up the good fight! 

:  [ 3:30 PM ][119] [ ![][13] ][120]

![][121][ ][122]

[Tim Finer][122] said... 
: 

Thanks for the article - I like it, now I need to read more about some of these other free tools that do the same. I can finally appreciate what the excitement of LISP like languages is all about. 

:  [ 9:08 PM ][123] [ ![][13] ][124]

![][125][ ![][126]][127]

[Victor Sergienko][127] said... 
: 

I also got a C++ background, loved it and finally lost my trust in it.  
  
Why not doing one more step and leaving C at all, for Python, Haskell, Lua or whatever? 

:  [ 4:16 PM ][128] [ ![][13] ][129]

![][130][ ![][131]][132]

[Serge Brunov][132] said... 
: 

Great article! 

:  [ 9:09 AM ][133] [ ![][13] ][134]

![][31]

Anonymous said... 
: 

[... ] is another relavant source of tips on this issue[...] 

:  [ 6:31 AM ][135] [ ![][13] ][136]

![][31]

Anonymous said... 
: 

Keep posting stuff like this i really like it 

:  [ 10:22 AM ][137] [ ![][13] ][138]

![][139][ ][140]

[Regan][140] said... 
: 

Hey, like the article. I've had many of the same experiences and come to many similar conclusions.  
  
Have you ever tried The D programming language (http://www.digitalmars.com/d).  
  
There is one thing which may put you off, and that is the built in garbage collector. However, it is predictable (only triggers on allocation) and can be disabled entirely. Plus there are plans to have several varieties and make it configurable.  
  
D has classes (single inheritance), interfaces, templates (but the syntax and implementation is vastly superior to C++) and there are no header files.   
  
Compile time speeds are 100x better than C/C++.  
  
Your RAII example can be handled by a 1 line scope(exit) statement.  
  
D has type safe printf/scanf.  
  
D is link time compatible with C, you can link D to C libraries, object files, etc.  
  
D plans to have, if it doesn't already, vector/array math.  
  
D has a feature called 'mixins' which work a bit like LISP macros injecting code into other code.  
  
Last but not least, the syntax is the same as C, and better yet the behaviour of statements in D matches those in C, intentionally, so transition is very easy. 

:  [ 11:18 AM ][141] [ ![][13] ][142]

![][143][ ][144]

[reroute][144] said... 
: 

You are my hero. I have been coding in Blub for a long time and have always wanted to do something like this (use Lisp macros and Lisp in general as a metaprogramming platform to spit out Blub architectures and abstractions). Thank you sir, I am inspired. 

:  [ 10:00 PM ][145] [ ![][13] ][146]

![][147][ ![][148]][149]

[Maxim Velesyuk][149] said... 
: 

There is a project by Fernando Borretti for C-style macros with macro expander written in Common Lisp: https://github.com/eudoxia0/cmacro, and a collection of basic macros: https://github.com/eudoxia0/magma  
  
This way he has already added lambdas, lazy evaluation, types, anaphoric macros, with- macros.  
  
For example, `unless` macro looks like this:  
macro unless {  
case {  
match {  
$(cond)  
}  
template {  
if(! $(cond))  
}  
}  
} 

:  [ 9:09 PM ][150] [ ![][13] ][151]

[Post a Comment][152]

[Newer Post][153] [Older Post][154] [Home][1]

Subscribe to: [Post Comments (Atom)][155]

## About Me

![My photo][156]

[ dep ][53]
: Software Engineer at EA DICE. Past projects include four of the Battlefield games. I'm currently working on the Frostbite game engine. I'm into portability, debugging and writing tools such as compilers and code generators.

[View my complete profile][53]

[ ![][157] ][158]

## Blog Archive

* [ ▼  ][159] [ 2010 ][160] (5)
    * [ ►  ][159] [ August ][161] (1)
    * [ ►  ][159] [ July ][162] (1)
    * [ ►  ][159] [ June ][163] (1)
    * [ ►  ][159] [ February ][164] (1)
    * [ ▼  ][159] [ January ][165] (1)
        * [Amplifying C][166]
* [ ►  ][159] [ 2009 ][167] (2)
    * [ ►  ][159] [ April ][168] (1)
    * [ ►  ][159] [ January ][169] (1)
* [ ►  ][159] [ 2008 ][170] (3)
    * [ ►  ][159] [ December ][171] (1)
    * [ ►  ][159] [ September ][172] (1)
    * [ ►  ][159] [ February ][173] (1)
* [ ►  ][159] [ 2007 ][174] (3)
    * [ ►  ][159] [ September ][175] (1)
    * [ ►  ][159] [ February ][176] (2)
* [ ►  ][159] [ 2006 ][177] (4)
    * [ ►  ][159] [ November ][178] (1)
    * [ ►  ][159] [ June ][179] (1)
    * [ ►  ][159] [ March ][180] (1)
    * [ ►  ][159] [ February ][181] (1)

[ ![][157] ][182]

| -----|  
|  

  |  

  |  

Simple theme. Powered by [Blogger][183]. 

[ ![][157] ][184]

[1]: http://voodoo-slide.blogspot.com/
[2]: https://www.blogger.com/profile/16229508667482062327 "author profile"
[3]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html "permanent link"
[4]: https://resources.blogblog.com/img/icon18_edit_allbkg.gif
[5]: https://www.blogger.com/post-edit.g?blogID=22374588&postID=1030759976264022429&from=pencil "Edit Post"
[6]: http://voodoo-slide.blogspot.com/search/label/c%2B%2B
[7]: http://voodoo-slide.blogspot.com/search/label/lisp
[8]: http://voodoo-slide.blogspot.com/search/label/rant
[9]: https://img1.blogblog.com/img/blank.gif "repi"
[10]: http://2.bp.blogspot.com/_TVtp2YyqLAw/SXqIt9ayuPI/AAAAAAAAAOQ/4eCJmLXXVds/S45-s35/repi_gecko.jpg
[11]: https://www.blogger.com/profile/03696561096253736133
[12]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264184243558#c4302798942379893271 "comment permalink"
[13]: https://resources.blogblog.com/img/icon_delete13.gif
[14]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=4302798942379893271 "Delete Comment"
[15]: https://img1.blogblog.com/img/blank.gif "Jonas"
[16]: http://1.bp.blogspot.com/_trhqpQIzL64/SuwUt1ir0lI/AAAAAAAAAbc/JA0nFOxq-qc/S45-s35/me.jpg
[17]: https://www.blogger.com/profile/12004246972597429066
[18]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264186538851#c7739759111190508404 "comment permalink"
[19]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=7739759111190508404 "Delete Comment"
[20]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "p10q"
[21]: https://www.blogger.com/profile/13184469130055785295
[22]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264190636706#c8237676571471561943 "comment permalink"
[23]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=8237676571471561943 "Delete Comment"
[24]: http://img1.blogblog.com/img/blank.gif "EvanM"
[25]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264207897913#c8563888538941330867 "comment permalink"
[26]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=8563888538941330867 "Delete Comment"
[27]: http://img1.blogblog.com/img/blank.gif "David Reynolds"
[28]: http://alwaysmovefast.com
[29]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264214967053#c6764289697953892746 "comment permalink"
[30]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=6764289697953892746 "Delete Comment"
[31]: http://img1.blogblog.com/img/blank.gif "Anonymous"
[32]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264215035655#c5190585672171290170 "comment permalink"
[33]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=5190585672171290170 "Delete Comment"
[34]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "malkia"
[35]: https://www.blogger.com/profile/07408170618420648277
[36]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264215949652#c7966835683254790018 "comment permalink"
[37]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=7966835683254790018 "Delete Comment"
[38]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Christopher Armstrong"
[39]: https://www.blogger.com/profile/11041638059246049826
[40]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264218375570#c1446154693140137841 "comment permalink"
[41]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=1446154693140137841 "Delete Comment"
[42]: http://img1.blogblog.com/img/blank.gif "Colin Curtin"
[43]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264219902370#c6452236285793679418 "comment permalink"
[44]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=6452236285793679418 "Delete Comment"
[45]: http://img1.blogblog.com/img/blank.gif "Scott"
[46]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264220693719#c1713312139672497303 "comment permalink"
[47]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=1713312139672497303 "Delete Comment"
[48]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "WebDevHobo"
[49]: https://www.blogger.com/profile/11231491634357108696
[50]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264235338321#c6647196991382490932 "comment permalink"
[51]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=6647196991382490932 "Delete Comment"
[52]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "dep"
[53]: https://www.blogger.com/profile/16229508667482062327
[54]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264237296011#c8787874162102862952 "comment permalink"
[55]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=8787874162102862952 "Delete Comment"
[56]: https://img1.blogblog.com/img/blank.gif "Jyaif"
[57]: http://4.bp.blogspot.com/_Pb5ghyz83zQ/S0dUgoJQI3I/AAAAAAAAAfE/GObcoInny8s/S45-s35/jf.jpeg
[58]: https://www.blogger.com/profile/07832250921952188906
[59]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264252046753#c2518048150406979227 "comment permalink"
[60]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=2518048150406979227 "Delete Comment"
[61]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Ali Sabil"
[62]: https://www.blogger.com/profile/11072457747541422509
[63]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264263423161#c5122151230967510916 "comment permalink"
[64]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=5122151230967510916 "Delete Comment"
[65]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "patrickdlogan"
[66]: https://www.blogger.com/profile/09030151653908100586
[67]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264264205117#c1562524976858171942 "comment permalink"
[68]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=1562524976858171942 "Delete Comment"
[69]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "eryx"
[70]: https://www.blogger.com/profile/12114059910167045752
[71]: http://super.para.media.kyoto-u.ac.jp/~tasuku/sc/index.html
[72]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264273876182#c1326663320906894757 "comment permalink"
[73]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=1326663320906894757 "Delete Comment"
[74]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "dimatura"
[75]: https://www.blogger.com/profile/08120351151753560577
[76]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264274098848#c1086632414252536491 "comment permalink"
[77]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=1086632414252536491 "Delete Comment"
[78]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264375183777#c1601251183001115668 "comment permalink"
[79]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=1601251183001115668 "Delete Comment"
[80]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Maybe Attending"
[81]: https://www.blogger.com/profile/16645849637325178047
[82]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264377409538#c2277788357974702181 "comment permalink"
[83]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=2277788357974702181 "Delete Comment"
[84]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Johan Kotlinski"
[85]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264377509989#c6299939551805313273 "comment permalink"
[86]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=6299939551805313273 "Delete Comment"
[87]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264392805267#c5617554425113126797 "comment permalink"
[88]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=5617554425113126797 "Delete Comment"
[89]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "temoto"
[90]: https://www.blogger.com/profile/03622566552770711788
[91]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264422971882#c3289366265055873454 "comment permalink"
[92]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=3289366265055873454 "Delete Comment"
[93]: http://img1.blogblog.com/img/blank.gif "NONAME"
[94]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264453299270#c8328631844365418361 "comment permalink"
[95]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=8328631844365418361 "Delete Comment"
[96]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Jason Messer"
[97]: https://www.blogger.com/profile/16582212440245620338
[98]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264488363602#c254885191290015943 "comment permalink"
[99]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=254885191290015943 "Delete Comment"
[100]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "rampelstinskin"
[101]: https://www.blogger.com/profile/00824271739785856031
[102]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264502898235#c5613535710728816508 "comment permalink"
[103]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=5613535710728816508 "Delete Comment"
[104]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Dimitris Menounos"
[105]: https://www.blogger.com/profile/07416863812258576754
[106]: http://live.gnome.org/Vala
[107]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264512876492#c8511553295683596016 "comment permalink"
[108]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=8511553295683596016 "Delete Comment"
[109]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264667105836#c225729269242471741 "comment permalink"
[110]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=225729269242471741 "Delete Comment"
[111]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Larry Clapp"
[112]: https://www.blogger.com/profile/09056791619989601236
[113]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264772378191#c1178762741623310631 "comment permalink"
[114]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=1178762741623310631 "Delete Comment"
[115]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264772471023#c4317426728806436091 "comment permalink"
[116]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=4317426728806436091 "Delete Comment"
[117]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "cl-beer"
[118]: https://www.blogger.com/profile/15131008286303003978
[119]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264775414742#c3805333122673063369 "comment permalink"
[120]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=3805333122673063369 "Delete Comment"
[121]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Tim Finer"
[122]: https://www.blogger.com/profile/09659969985303664340
[123]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1264968485900#c2485491650787502079 "comment permalink"
[124]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=2485491650787502079 "Delete Comment"
[125]: https://img1.blogblog.com/img/blank.gif "Victor Sergienko"
[126]: http://3.bp.blogspot.com/_mV7m4IFgIYY/Sap6PLtK5CI/AAAAAAAAAhc/QcBdP3JD4ok/S45-s35/vise.jpg
[127]: https://www.blogger.com/profile/03248158339739606306
[128]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1265383010031#c1020144410937114824 "comment permalink"
[129]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=1020144410937114824 "Delete Comment"
[130]: https://img1.blogblog.com/img/blank.gif "Serge Brunov"
[131]: http://3.bp.blogspot.com/_3RRFQNW9lRc/Suq05rBoiaI/AAAAAAAAAA0/mkXE9XYMOi8/S45-s35/domovoy1_crop.jpg
[132]: https://www.blogger.com/profile/18151369832103868620
[133]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1265616558831#c3271490091316302858 "comment permalink"
[134]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=3271490091316302858 "Delete Comment"
[135]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1266211885249#c2750883010714588176 "comment permalink"
[136]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=2750883010714588176 "Delete Comment"
[137]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1266571336849#c5286655811931504968 "comment permalink"
[138]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=5286655811931504968 "Delete Comment"
[139]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "Regan"
[140]: https://www.blogger.com/profile/06000308959855315673
[141]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1298542696316#c6910490102713470225 "comment permalink"
[142]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=6910490102713470225 "Delete Comment"
[143]: http://lh3.googleusercontent.com/zFdxGE77vvD2w5xHy6jkVuElKv-U9_9qLkRYK8OnbDeJPtjSZ82UPq5w6hJ-SA%3Ds35 "reroute"
[144]: https://www.blogger.com/profile/17485951477005558322
[145]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1347652816038#c1675674055739493889 "comment permalink"
[146]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=1675674055739493889 "Delete Comment"
[147]: https://img1.blogblog.com/img/blank.gif "Maxim Velesyuk"
[148]: http://lh6.googleusercontent.com/-PBycnlt0whM/AAAAAAAAAAI/AAAAAAAAACg/pWNo0yYoDDg/s35-c/photo.jpg
[149]: https://www.blogger.com/profile/17830188390738462013
[150]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html?showComment=1454875767154#c3228267744198813246 "comment permalink"
[151]: https://www.blogger.com/delete-comment.g?blogID=22374588&postID=3228267744198813246 "Delete Comment"
[152]: https://www.blogger.com/comment.g?blogID=22374588&postID=1030759976264022429
[153]: http://voodoo-slide.blogspot.com/2010/02/c-amplify-prototype-available.html "Newer Post"
[154]: http://voodoo-slide.blogspot.com/2009/04/amiga-remote-launching.html "Older Post"
[155]: http://voodoo-slide.blogspot.com/feeds/1030759976264022429/comments/default
[156]: http://1.bp.blogspot.com/_2P1ZGKDPDM0/TH-bhLwfvlI/AAAAAAAABAE/3X2QogezSUM/S220-s80/profile-blur.jpg
[157]: https://resources.blogblog.com/img/icon18_wrench_allbkg.png
[158]: //www.blogger.com/rearrange?blogID=22374588&widgetType=Profile&widgetId=Profile1&action=editWidget&sectionId=sidebar-right-1 "Edit"
[159]: javascript:void(0)
[160]: http://voodoo-slide.blogspot.com/2010/
[161]: http://voodoo-slide.blogspot.com/2010/08/
[162]: http://voodoo-slide.blogspot.com/2010/07/
[163]: http://voodoo-slide.blogspot.com/2010/06/
[164]: http://voodoo-slide.blogspot.com/2010/02/
[165]: http://voodoo-slide.blogspot.com/2010/01/
[166]: http://voodoo-slide.blogspot.com/2010/01/amplifying-c.html
[167]: http://voodoo-slide.blogspot.com/2009/
[168]: http://voodoo-slide.blogspot.com/2009/04/
[169]: http://voodoo-slide.blogspot.com/2009/01/
[170]: http://voodoo-slide.blogspot.com/2008/
[171]: http://voodoo-slide.blogspot.com/2008/12/
[172]: http://voodoo-slide.blogspot.com/2008/09/
[173]: http://voodoo-slide.blogspot.com/2008/02/
[174]: http://voodoo-slide.blogspot.com/2007/
[175]: http://voodoo-slide.blogspot.com/2007/09/
[176]: http://voodoo-slide.blogspot.com/2007/02/
[177]: http://voodoo-slide.blogspot.com/2006/
[178]: http://voodoo-slide.blogspot.com/2006/11/
[179]: http://voodoo-slide.blogspot.com/2006/06/
[180]: http://voodoo-slide.blogspot.com/2006/03/
[181]: http://voodoo-slide.blogspot.com/2006/02/
[182]: //www.blogger.com/rearrange?blogID=22374588&widgetType=BlogArchive&widgetId=BlogArchive1&action=editWidget&sectionId=sidebar-right-1 "Edit"
[183]: https://www.blogger.com
[184]: //www.blogger.com/rearrange?blogID=22374588&widgetType=Attribution&widgetId=Attribution1&action=editWidget&sectionId=footer-3 "Edit"

  [*[5:26 PM]: 2010-01-22T17:26:00+01:00

