---
created_at: '2014-11-23T09:39:50.000Z'
title: Go's Declaration Syntax (2010)
url: http://blog.golang.org/gos-declaration-syntax
author: dsr12
points: 86
story_text: ''
comment_text: 
num_comments: 40
story_id: 
story_title: 
story_url: 
parent_id: 
created_at_i: 1416735590
_tags:
- story
- author_dsr12
- story_8648280
objectID: '8648280'

---
[Source](https://blog.golang.org/gos-declaration-syntax "Permalink to Go&#39;s Declaration Syntax - The Go Blog")

# Go&#39;s Declaration Syntax - The Go Blog

[The Go Programming Language][1]

[Go][1]

▽

[Documents][2] [Packages][3] [The Project][4] [Help][5] [Blog][6] submit search

#### Next article

[Share Memory By Communicating][7]

#### Previous article

[Go Programming session video from Google I/O][8]

#### Links

* [golang.org][1]
* [Install Go][9]
* [A Tour of Go][10]
* [Go Documentation][2]
* [Go Mailing List][11]
* [Go on Google+][12]
* [Go+ Community][13]
* [Go on Twitter][14]

[Blog index][15]

# [The Go Blog][6]

### [Go's Declaration Syntax][16]

7 July 2010

#### Introduction

Newcomers to Go wonder why the declaration syntax is different from the tradition established in the C family. In this post we'll compare the two approaches and explain why Go's declarations look as they do. 

#### C syntax

First, let's talk about C syntax. C took an unusual and clever approach to declaration syntax. Instead of describing the types with special syntax, one writes an expression involving the item being declared, and states what type that expression will have. Thus 
    
    
    int x;

declares x to be an int: the expression 'x' will have type int. In general, to figure out how to write the type of a new variable, write an expression involving that variable that evaluates to a basic type, then put the basic type on the left and the expression on the right. 

Thus, the declarations 
    
    
    int *p;
    int a[3];

state that p is a pointer to int because '*p' has type int, and that a is an array of ints because a[3] (ignoring the particular index value, which is punned to be the size of the array) has type int. 

What about functions? Originally, C's function declarations wrote the types of the arguments outside the parens, like this: 
    
    
    int main(argc, argv)
        int argc;
        char *argv[];
    { /* ... */ }

Again, we see that main is a function because the expression main(argc, argv) returns an int. In modern notation we'd write 
    
    
    int main(int argc, char *argv[]) { /* ... */ }

but the basic structure is the same. 

This is a clever syntactic idea that works well for simple types but can get confusing fast. The famous example is declaring a function pointer. Follow the rules and you get this: 
    
    
    int (*fp)(int a, int b);

Here, fp is a pointer to a function because if you write the expression (*fp)(a, b) you'll call a function that returns int. What if one of fp's arguments is itself a function? 
    
    
    int (*fp)(int (*ff)(int x, int y), int b)

That's starting to get hard to read. 

Of course, we can leave out the name of the parameters when we declare a function, so main can be declared 
    
    
    int main(int, char *[])

Recall that argv is declared like this, 
    
    
    char *argv[]

so you drop the name from the middle of its declaration to construct its type. It's not obvious, though, that you declare something of type char *[] by putting its name in the middle. 

And look what happens to fp's declaration if you don't name the parameters: 
    
    
    int (*fp)(int (*)(int, int), int)

Not only is it not obvious where to put the name inside 
    
    
    int (*)(int, int)

it's not exactly clear that it's a function pointer declaration at all. And what if the return type is a function pointer? 
    
    
    int (*(*fp)(int (*)(int, int), int))(int, int)

It's hard even to see that this declaration is about fp. 

You can construct more elaborate examples but these should illustrate some of the difficulties that C's declaration syntax can introduce. 

There's one more point that needs to be made, though. Because type and declaration syntax are the same, it can be difficult to parse expressions with types in the middle. This is why, for instance, C casts always parenthesize the type, as in 
    
    
    (int)M_PI

#### Go syntax

Languages outside the C family usually use a distinct type syntax in declarations. Although it's a separate point, the name usually comes first, often followed by a colon. Thus our examples above become something like (in a fictional but illustrative language) 
    
    
    x: int
    p: pointer to int
    a: array[3] of int

These declarations are clear, if verbose - you just read them left to right. Go takes its cue from here, but in the interests of brevity it drops the colon and removes some of the keywords: 
    
    
    x int
    p *int
    a [3]int

There is no direct correspondence between the look of [3]int and how to use a in an expression. (We'll come back to pointers in the next section.) You gain clarity at the cost of a separate syntax. 

Now consider functions. Let's transcribe the declaration for main as it would read in Go, although the real main function in Go takes no arguments: 
    
    
    func main(argc int, argv []string) int

Superficially that's not much different from C, other than the change from `char` arrays to strings, but it reads well from left to right: 

function main takes an int and a slice of strings and returns an int. 

Drop the parameter names and it's just as clear - they're always first so there's no confusion. 
    
    
    func main(int, []string) int

One merit of this left-to-right style is how well it works as the types become more complex. Here's a declaration of a function variable (analogous to a function pointer in C): 
    
    
    f func(func(int,int) int, int) int

Or if f returns a function: 
    
    
    f func(func(int,int) int, int) func(int, int) int

It still reads clearly, from left to right, and it's always obvious which name is being declared - the name comes first. 

The distinction between type and expression syntax makes it easy to write and invoke closures in Go: 
    
    
    sum := func(a, b int) int { return a+b } (3, 4)

#### Pointers

Pointers are the exception that proves the rule. Notice that in arrays and slices, for instance, Go's type syntax puts the brackets on the left of the type but the expression syntax puts them on the right of the expression: 
    
    
    var a []int
    x = a[1]

For familiarity, Go's pointers use the * notation from C, but we could not bring ourselves to make a similar reversal for pointer types. Thus pointers work like this 
    
    
    var p *int
    x = *p

We couldn't say 
    
    
    var p *int
    x = p*

because that postfix * would conflate with multiplication. We could have used the Pascal ^, for example: 
    
    
    var p ^int
    x = p^

and perhaps we should have (and chosen another operator for xor), because the prefix asterisk on both types and expressions complicates things in a number of ways. For instance, although one can write 
    
    
    []int("hi")

as a conversion, one must parenthesize the type if it starts with a *: 
    
    
    (*int)(nil)

Had we been willing to give up * as pointer syntax, those parentheses would be unnecessary. 

So Go's pointer syntax is tied to the familiar C form, but those ties mean that we cannot break completely from using parentheses to disambiguate types and expressions in the grammar. 

Overall, though, we believe Go's type syntax is easier to understand than C's, especially when things get complicated. 

#### Notes

Go's declarations read left to right. It's been pointed out that C's read in a spiral! See [ The "Clockwise/Spiral Rule"][17] by David Anderson. 

By Rob Pike

## Related articles

* [Two recent Go articles][18]
* [Two recent Go talks][19]
* [Go videos from Google I/O 2012][20]

Except as [noted][21], the content of this page is licensed under the Creative Commons Attribution 3.0 License,  
and code is licensed under a [BSD license][22].  
[Terms of Service][23] | [Privacy Policy][24] | [View the source code][25]

[1]: https://golang.org/
[2]: https://golang.org/doc/
[3]: https://golang.org/pkg/
[4]: https://golang.org/project/
[5]: https://golang.org/help/
[6]: https://blog.golang.org/
[7]: https://blog.golang.org/share-memory-by-communicating
[8]: https://blog.golang.org/go-programming-session-video-from
[9]: https://golang.org/doc/install.html
[10]: https://blog.golang.org//tour.golang.org/
[11]: https://blog.golang.org//groups.google.com/group/golang-nuts
[12]: https://blog.golang.org//plus.google.com/101406623878176903605
[13]: https://blog.golang.org//plus.google.com/112164155169467723645/posts
[14]: https://blog.golang.org//twitter.com/golang
[15]: https://blog.golang.org/index
[16]: https://blog.golang.org/gos-declaration-syntax
[17]: http://c-faq.com/decl/spiral.anderson.html
[18]: https://blog.golang.org/two-recent-go-articles
[19]: https://blog.golang.org/two-recent-go-talks
[20]: https://blog.golang.org/go-videos-from-google-io-2012
[21]: https://developers.google.com/site-policies#restrictions
[22]: https://golang.org/LICENSE
[23]: https://golang.org/doc/tos.html
[24]: https://blog.golang.org//www.google.com/intl/en/policies/privacy/
[25]: https://go.googlesource.com/blog/

